\\ Copyright 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


\\ Usage: gp <Hironaka-curve.gp
\\
\\ A form of Hironaka's curve can be constructed from the ternary digits of
\\ a fraction f in the range 0 <= f <= 1, transformed to signed binary bits,
\\
\\       f  = . 0 2 2 0 1 0  1  1 2 0 0  1 1 2 2 ...   ternary
\\     H(f) = . 0 0 0 0 1 0 -1  1 0 0 0 -1 1 0 0 ...   binary
\\
\\ Each ternary 0 or 2 becomes binary 0.
\\ Each ternary 1 becomes binary 1 or -1 with alternating signs.
\\ This is equivalent to the sign determined by whether an even or odd
\\ number of ternary 1 digits above.
\\

default(strictargs,1);
default(recover,0);
read("../vecper.gp");


real_to_frac(f) =
{
  if(type(f)=="t_REAL",
     \\ binary() gives [ [bits above point], [bits below point] ]
     \\ or 0.0 is just [0,...,0]
     my(v=binary(f));
     if(f,subst(Pol(v[2]),'x,2)) \\ like fromdigits(), but work in gp 2.7.x
     / 2^#v[2];
     ,
     f);
}

\\ f is a fraction in the range 0 <= f <= 1.
\\ Return a2-element vector [new_f, d] where d = 0,1,2 is the ternary digit
\\ of f immediately below the radix point, and new_f is the remainder,
\\    f == 1/3*(d + new_f)
\\ For f=1, the return has d=2 (and new_f=1), as for 1 = .2222...
\\
take_ternary_digit(f) =
{
  (0 <= f && f <= 1) || error("oops, f out of range: ",f);
  f*=3;
  my(d=min(2,floor(f)));
  [f-d, d];
}

\\ Convert a fraction f to a vecper of signed binary bits.
Hironaka_f_to_vecper(f) =
{
  f=real_to_frac(f);

  \\ "state" in a 2-element vector [f_remaining, sign +1 or -1].
  vecper_make_state
    ((state)->
     my(f,s,d); [f,s]=state;
     [f,d]=take_ternary_digit(f);
     d = (d%2)*s;      \\ 0,2 -> 0,  1 -> s
     if(d, s=-s);      \\ negate s below a ternary 1 digit
     [d, [f,s]];
     ,
     [f,1]);   \\ initial whole f, sign +1
}
Hironaka_f_to_y(f) = vecper_frac_fromdigits(Hironaka_f_to_vecper(f), 2);

\\ 0 = .000 and 1 = .222 both become binary .000
Hironaka_f_to_vecper(0)   == [[], [0]]   || error();
Hironaka_f_to_vecper(1)   == [[], [0]]   || error();
Hironaka_f_to_y(0)   == 0                || error();
Hironaka_f_to_y(1)   == 0                || error();

\\ 1/2 = .111 becomes alternating signs binary . 1 -1 1 -1
Hironaka_f_to_vecper(1/2) == [[], [1,-1]]   || error();
Hironaka_f_to_y(1/2) == 1/3                 || error();


\\-----------------------------------------------------------------------------

{
  my(len=3^5,
     \\ f values 0/len, 1/len, ..., len/len
     f_vec=vector(len+1,i, (i-1)/len),
     y_vec=apply(Hironaka_f_to_y, f_vec));
\\ print(f_vec);
\\ print(y_vec);
  print("y range "vecmin(y_vec)" .. "vecmax(y_vec));
  plothraw(f_vec,y_vec, 1);
}

\\-----------------------------------------------------------------------------
